# commands:generator
from .main import generate
from .main import admin
from .main import form
from .main import manager
from .main import model
from .main import resource
from .main import serializer
from .main import sql_view
from .main import template
from .main import test
from .main import view
from .main import viewset
