# Generated by django-clite

migrations.RunSQL(
    """
    DROP VIEW IF EXISTS {{ db_table.lower() }};
    CREATE OR REPLACE VIEW {{ db_table.lower() }} AS
        SELECT
            {% for field in fields %}
                {{ field }}
            {% endfor %}
        FROM {{ source_db }};
    """
)